@@ -24,6 +24,7 @@
 #include "qgspolygon.h"
 #include "qgswkbptr.h"
 #include "qgsmulticurve.h"
+#include "qgsfeedback.h"
 
 #include <QJsonArray>
 #include <QJsonObject>
@@ -34,7 +35,7 @@
 
 QgsCurvePolygon::QgsCurvePolygon()
 {
-  mWkbType = QgsWkbTypes::CurvePolygon;
+  mWkbType = Qgis::WkbType::CurvePolygon;
 }
 
 QgsCurvePolygon::~QgsCurvePolygon()
@@ -44,7 +45,7 @@ QgsCurvePolygon::~QgsCurvePolygon()
 
 QgsCurvePolygon *QgsCurvePolygon::createEmptyWithSameType() const
 {
-  auto result = qgis::make_unique< QgsCurvePolygon >();
+  auto result = std::make_unique< QgsCurvePolygon >();
   result->mWkbType = mWkbType;
   return result.release();
 }
@@ -73,6 +74,10 @@ QgsCurvePolygon::QgsCurvePolygon( const QgsCurvePolygon &p )
   {
     mInteriorRings.push_back( ring->clone() );
   }
+
+  mBoundingBox = p.mBoundingBox;
+  mHasCachedValidity = p.mHasCachedValidity;
+  mValidityFailureReason = p.mValidityFailureReason;
 }
 
 QgsCurvePolygon &QgsCurvePolygon::operator=( const QgsCurvePolygon &p )
@@ -143,7 +148,7 @@ QgsCurvePolygon *QgsCurvePolygon::clone() const
 
 void QgsCurvePolygon::clear()
 {
-  mWkbType = QgsWkbTypes::CurvePolygon;
+  mWkbType = Qgis::WkbType::CurvePolygon;
   mExteriorRing.reset();
   qDeleteAll( mInteriorRings );
   mInteriorRings.clear();
@@ -159,8 +164,8 @@ bool QgsCurvePolygon::fromWkb( QgsConstWkbPtr &wkbPtr )
     return false;
   }
 
-  QgsWkbTypes::Type type = wkbPtr.readHeader();
-  if ( QgsWkbTypes::flatType( type ) != QgsWkbTypes::CurvePolygon )
+  Qgis::WkbType type = wkbPtr.readHeader();
+  if ( QgsWkbTypes::flatType( type ) != Qgis::WkbType::CurvePolygon )
   {
     return false;
   }
@@ -171,18 +176,18 @@ bool QgsCurvePolygon::fromWkb( QgsConstWkbPtr &wkbPtr )
   std::unique_ptr< QgsCurve > currentCurve;
   for ( int i = 0; i < nRings; ++i )
   {
-    QgsWkbTypes::Type curveType = wkbPtr.readHeader();
+    Qgis::WkbType curveType = wkbPtr.readHeader();
     wkbPtr -= 1 + sizeof( int );
-    QgsWkbTypes::Type flatCurveType = QgsWkbTypes::flatType( curveType );
-    if ( flatCurveType == QgsWkbTypes::LineString )
+    Qgis::WkbType flatCurveType = QgsWkbTypes::flatType( curveType );
+    if ( flatCurveType == Qgis::WkbType::LineString )
     {
       currentCurve.reset( new QgsLineString() );
     }
-    else if ( flatCurveType == QgsWkbTypes::CircularString )
+    else if ( flatCurveType == Qgis::WkbType::CircularString )
     {
       currentCurve.reset( new QgsCircularString() );
     }
-    else if ( flatCurveType == QgsWkbTypes::CompoundCurve )
+    else if ( flatCurveType == Qgis::WkbType::CompoundCurve )
     {
       currentCurve.reset( new QgsCompoundCurve() );
     }
@@ -208,9 +213,9 @@ bool QgsCurvePolygon::fromWkt( const QString &wkt )
 {
   clear();
 
-  QPair<QgsWkbTypes::Type, QString> parts = QgsGeometryUtils::wktReadBlock( wkt );
+  QPair<Qgis::WkbType, QString> parts = QgsGeometryUtils::wktReadBlock( wkt );
 
-  if ( QgsWkbTypes::geometryType( parts.first ) != QgsWkbTypes::PolygonGeometry )
+  if ( QgsWkbTypes::geometryType( parts.first ) != Qgis::GeometryType::Polygon )
     return false;
 
   mWkbType = parts.first;
@@ -226,14 +231,14 @@ bool QgsCurvePolygon::fromWkt( const QString &wkt )
   const QStringList blocks = QgsGeometryUtils::wktGetChildBlocks( parts.second, defaultChildWkbType );
   for ( const QString &childWkt : blocks )
   {
-    QPair<QgsWkbTypes::Type, QString> childParts = QgsGeometryUtils::wktReadBlock( childWkt );
+    QPair<Qgis::WkbType, QString> childParts = QgsGeometryUtils::wktReadBlock( childWkt );
 
-    QgsWkbTypes::Type flatCurveType = QgsWkbTypes::flatType( childParts.first );
-    if ( flatCurveType == QgsWkbTypes::LineString )
+    Qgis::WkbType flatCurveType = QgsWkbTypes::flatType( childParts.first );
+    if ( flatCurveType == Qgis::WkbType::LineString )
       mInteriorRings.append( new QgsLineString() );
-    else if ( flatCurveType == QgsWkbTypes::CircularString )
+    else if ( flatCurveType == Qgis::WkbType::CircularString )
       mInteriorRings.append( new QgsCircularString() );
-    else if ( flatCurveType == QgsWkbTypes::CompoundCurve )
+    else if ( flatCurveType == Qgis::WkbType::CompoundCurve )
       mInteriorRings.append( new QgsCompoundCurve() );
     else
     {
@@ -264,7 +269,7 @@ bool QgsCurvePolygon::fromWkt( const QString &wkt )
     hasZ = hasZ || mExteriorRing->is3D();
     hasM = hasM || mExteriorRing->isMeasure();
   }
-  for ( const QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( const QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     hasZ = hasZ || curve->is3D();
     hasM = hasM || curve->isMeasure();
@@ -279,13 +284,13 @@ bool QgsCurvePolygon::fromWkt( const QString &wkt )
   return true;
 }
 
-QgsRectangle QgsCurvePolygon::calculateBoundingBox() const
+QgsBox3D QgsCurvePolygon::calculateBoundingBox3D() const
 {
   if ( mExteriorRing )
   {
-    return mExteriorRing->boundingBox();
+    return mExteriorRing->boundingBox3D();
   }
-  return QgsRectangle();
+  return QgsBox3D();
 }
 
 int QgsCurvePolygon::wkbSize( QgsAbstractGeometry::WkbFlags flags ) const
@@ -463,6 +468,27 @@ QString QgsCurvePolygon::asKml( int precision ) const
   return kml;
 }
 
+void QgsCurvePolygon::normalize()
+{
+  // normalize rings
+  if ( mExteriorRing )
+    mExteriorRing->normalize();
+
+  for ( QgsCurve *ring : std::as_const( mInteriorRings ) )
+  {
+    ring->normalize();
+  }
+
+  // sort rings
+  std::sort( mInteriorRings.begin(), mInteriorRings.end(), []( const QgsCurve * a, const QgsCurve * b )
+  {
+    return a->compareTo( b ) > 0;
+  } );
+
+  // normalize ring orientation
+  forceRHR();
+}
+
 double QgsCurvePolygon::area() const
 {
   if ( !mExteriorRing )
@@ -505,6 +531,15 @@ double QgsCurvePolygon::perimeter() const
   return perimeter;
 }
 
+double QgsCurvePolygon::roundness() const
+{
+  const double p = perimeter();
+  if ( qgsDoubleNear( p, 0.0 ) )
+    return 0.0;
+
+  return 4.0 * M_PI * area() / pow( p, 2.0 );
+}
+
 QgsPolygon *QgsCurvePolygon::surfaceToPolygon() const
 {
   std::unique_ptr< QgsPolygon > polygon( new QgsPolygon() );
@@ -591,7 +626,7 @@ bool QgsCurvePolygon::removeDuplicateNodes( double epsilon, bool useZValues )
     if ( ring->removeDuplicateNodes( epsilon, useZValues ) )
     {
       QgsPoint startPoint;
-      QgsVertexId::VertexType type;
+      Qgis::VertexType type;
       ring->pointAt( 0, startPoint, type );
       // ensure ring is properly closed - if we removed the final node, it may no longer be properly closed
       ring->moveVertex( QgsVertexId( -1, -1, ring->numPoints() - 1 ), startPoint );
@@ -604,13 +639,46 @@ bool QgsCurvePolygon::removeDuplicateNodes( double epsilon, bool useZValues )
   {
     result = cleanRing( mExteriorRing.get() );
   }
-  for ( QgsCurve *ring : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *ring : std::as_const( mInteriorRings ) )
   {
     if ( cleanRing( ring ) ) result = true;
   }
   return result;
 }
 
+bool QgsCurvePolygon::boundingBoxIntersects( const QgsBox3D &box3d ) const
+{
+  if ( !mExteriorRing && mInteriorRings.empty() )
+    return false;
+
+  // if we already have the bounding box calculated, then this check is trivial!
+  if ( !mBoundingBox.isNull() )
+  {
+    return mBoundingBox.intersects( box3d );
+  }
+
+  // loop through each ring and test the bounding box intersection.
+  // This gives us a chance to use optimisations which may be present on the individual
+  // ring geometry subclasses, and at worst it will cause a calculation of the bounding box
+  // of each individual ring geometry which we would have to do anyway... (and these
+  // bounding boxes are cached, so would be reused without additional expense)
+  if ( mExteriorRing && mExteriorRing->boundingBoxIntersects( box3d ) )
+    return true;
+
+  for ( const QgsCurve *ring : mInteriorRings )
+  {
+    if ( ring->boundingBoxIntersects( box3d ) )
+      return true;
+  }
+
+  // even if we don't intersect the bounding box of any rings, we may still intersect the
+  // bounding box of the overall polygon (we are considering worst case scenario here and
+  // the polygon is invalid, with rings outside the exterior ring!)
+  // so here we fall back to the non-optimised base class check which has to first calculate
+  // the overall bounding box of the polygon..
+  return QgsSurface::boundingBoxIntersects( box3d );
+}
+
 QgsPolygon *QgsCurvePolygon::toPolygon( double tolerance, SegmentationToleranceType toleranceType ) const
 {
   std::unique_ptr< QgsPolygon > poly( new QgsPolygon() );
@@ -640,17 +708,17 @@ void QgsCurvePolygon::setExteriorRing( QgsCurve *ring )
   mExteriorRing.reset( ring );
 
   //set proper wkb type
-  if ( QgsWkbTypes::flatType( wkbType() ) == QgsWkbTypes::Polygon )
+  if ( QgsWkbTypes::flatType( wkbType() ) == Qgis::WkbType::Polygon )
   {
-    setZMTypeFromSubGeometry( ring, QgsWkbTypes::Polygon );
+    setZMTypeFromSubGeometry( ring, Qgis::WkbType::Polygon );
   }
-  else if ( QgsWkbTypes::flatType( wkbType() ) == QgsWkbTypes::CurvePolygon )
+  else if ( QgsWkbTypes::flatType( wkbType() ) == Qgis::WkbType::CurvePolygon )
   {
-    setZMTypeFromSubGeometry( ring, QgsWkbTypes::CurvePolygon );
+    setZMTypeFromSubGeometry( ring, Qgis::WkbType::CurvePolygon );
   }
 
   //match dimensionality for rings
-  for ( QgsCurve *ring : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *ring : std::as_const( mInteriorRings ) )
   {
     if ( is3D() )
       ring->addZValue();
@@ -719,7 +787,7 @@ void QgsCurvePolygon::removeInteriorRings( double minimumAllowedArea )
     {
       double area = 0.0;
       mInteriorRings.at( ringIndex )->sumUpArea( area );
-      if ( area < minimumAllowedArea )
+      if ( std::fabs( area ) < minimumAllowedArea )
         delete mInteriorRings.takeAt( ringIndex );
     }
   }
@@ -731,7 +799,7 @@ void QgsCurvePolygon::removeInvalidRings()
 {
   QVector<QgsCurve *> validRings;
   validRings.reserve( mInteriorRings.size() );
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     if ( !curve->isRing() )
     {
@@ -748,7 +816,12 @@ void QgsCurvePolygon::removeInvalidRings()
 
 void QgsCurvePolygon::forceRHR()
 {
-  if ( mExteriorRing && mExteriorRing->orientation() != QgsCurve::Clockwise )
+  forceClockwise();
+}
+
+void QgsCurvePolygon::forceClockwise()
+{
+  if ( mExteriorRing && mExteriorRing->orientation() != Qgis::AngularDirection::Clockwise )
   {
     // flip exterior ring orientation
     std::unique_ptr< QgsCurve > flipped( mExteriorRing->reversed() );
@@ -756,9 +829,35 @@ void QgsCurvePolygon::forceRHR()
   }
 
   QVector<QgsCurve *> validRings;
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
-    if ( curve && curve->orientation() != QgsCurve::CounterClockwise )
+    if ( curve && curve->orientation() != Qgis::AngularDirection::CounterClockwise )
+    {
+      // flip interior ring orientation
+      QgsCurve *flipped = curve->reversed();
+      validRings << flipped;
+      delete curve;
+    }
+    else
+    {
+      validRings << curve;
+    }
+  }
+  mInteriorRings = validRings;
+}
+
+void QgsCurvePolygon::forceCounterClockwise()
+{
+  if ( mExteriorRing && mExteriorRing->orientation() != Qgis::AngularDirection::CounterClockwise )
+  {
+    // flip exterior ring orientation
+    mExteriorRing.reset( mExteriorRing->reversed() );
+  }
+
+  QVector<QgsCurve *> validRings;
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
+  {
+    if ( curve && curve->orientation() != Qgis::AngularDirection::Clockwise )
     {
       // flip interior ring orientation
       QgsCurve *flipped = curve->reversed();
@@ -815,14 +914,14 @@ void QgsCurvePolygon::draw( QPainter &p ) const
   }
 }
 
-void QgsCurvePolygon::transform( const QgsCoordinateTransform &ct, QgsCoordinateTransform::TransformDirection d, bool transformZ )
+void QgsCurvePolygon::transform( const QgsCoordinateTransform &ct, Qgis::TransformDirection d, bool transformZ )
 {
   if ( mExteriorRing )
   {
     mExteriorRing->transform( ct, d, transformZ );
   }
 
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->transform( ct, d, transformZ );
   }
@@ -836,7 +935,7 @@ void QgsCurvePolygon::transform( const QTransform &t, double zTranslate, double
     mExteriorRing->transform( t, zTranslate, zScale, mTranslate, mScale );
   }
 
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->transform( t, zTranslate, zScale, mTranslate, mScale );
   }
@@ -885,11 +984,11 @@ int QgsCurvePolygon::vertexNumberFromVertexId( QgsVertexId id ) const
   if ( id.part != 0 )
     return -1;
 
-  if ( id.ring < 0 || id.ring >= ringCount() )
+  if ( id.ring < 0 || id.ring >= ringCount() || !mExteriorRing )
     return -1;
 
   int number = 0;
-  if ( id.ring == 0 && mExteriorRing )
+  if ( id.ring == 0 )
   {
     return mExteriorRing->vertexNumberFromVertexId( QgsVertexId( 0, 0, id.vertex ) );
   }
@@ -1076,12 +1175,14 @@ bool QgsCurvePolygon::moveVertex( QgsVertexId vId, const QgsPoint &newPos )
 
 bool QgsCurvePolygon::deleteVertex( QgsVertexId vId )
 {
-  if ( !mExteriorRing || vId.ring < 0 || vId.ring >= 1 + mInteriorRings.size() )
+  const int interiorRingId = vId.ring - 1;
+  if ( !mExteriorRing || vId.ring < 0 || interiorRingId >= mInteriorRings.size() )
   {
     return false;
   }
 
-  QgsCurve *ring = vId.ring == 0 ? mExteriorRing.get() : mInteriorRings.at( vId.ring - 1 );
+  // cppcheck-suppress containerOutOfBounds
+  QgsCurve *ring = vId.ring == 0 ? mExteriorRing.get() : mInteriorRings.at( interiorRingId );
   int n = ring->numPoints();
   if ( n <= 4 )
   {
@@ -1191,7 +1292,7 @@ bool QgsCurvePolygon::addZValue( double zValue )
 
   if ( mExteriorRing )
     mExteriorRing->addZValue( zValue );
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->addZValue( zValue );
   }
@@ -1208,7 +1309,7 @@ bool QgsCurvePolygon::addMValue( double mValue )
 
   if ( mExteriorRing )
     mExteriorRing->addMValue( mValue );
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->addMValue( mValue );
   }
@@ -1224,7 +1325,7 @@ bool QgsCurvePolygon::dropZValue()
   mWkbType = QgsWkbTypes::dropZ( mWkbType );
   if ( mExteriorRing )
     mExteriorRing->dropZValue();
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->dropZValue();
   }
@@ -1240,7 +1341,7 @@ bool QgsCurvePolygon::dropMValue()
   mWkbType = QgsWkbTypes::dropM( mWkbType );
   if ( mExteriorRing )
     mExteriorRing->dropMValue();
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->dropMValue();
   }
@@ -1252,7 +1353,7 @@ void QgsCurvePolygon::swapXy()
 {
   if ( mExteriorRing )
     mExteriorRing->swapXy();
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->swapXy();
   }
@@ -1264,12 +1365,41 @@ QgsCurvePolygon *QgsCurvePolygon::toCurveType() const
   return clone();
 }
 
+bool QgsCurvePolygon::transform( QgsAbstractGeometryTransformer *transformer, QgsFeedback *feedback )
+{
+  if ( !transformer )
+    return false;
+
+  bool res = true;
+  if ( mExteriorRing )
+    res = mExteriorRing->transform( transformer, feedback );
+
+  if ( !res || ( feedback && feedback->isCanceled() ) )
+  {
+    clearCache();
+    return false;
+  }
+
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
+  {
+    res = curve->transform( transformer );
+
+    if ( feedback && feedback->isCanceled() )
+      res = false;
+
+    if ( !res )
+      break;
+  }
+  clearCache();
+  return res;
+}
+
 void QgsCurvePolygon::filterVertices( const std::function<bool ( const QgsPoint & )> &filter )
 {
   if ( mExteriorRing )
     mExteriorRing->filterVertices( filter );
 
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->filterVertices( filter );
   }
@@ -1281,7 +1411,7 @@ void QgsCurvePolygon::transformVertices( const std::function<QgsPoint( const Qgs
   if ( mExteriorRing )
     mExteriorRing->transformVertices( transform );
 
-  for ( QgsCurve *curve : qgis::as_const( mInteriorRings ) )
+  for ( QgsCurve *curve : std::as_const( mInteriorRings ) )
   {
     curve->transformVertices( transform );
   }
@@ -1300,3 +1430,45 @@ QgsAbstractGeometry *QgsCurvePolygon::childGeometry( int index ) const
   else
     return mInteriorRings.at( index - 1 );
 }
+
+int QgsCurvePolygon::compareToSameClass( const QgsAbstractGeometry *other ) const
+{
+  const QgsCurvePolygon *otherPolygon = qgsgeometry_cast<const QgsCurvePolygon *>( other );
+  if ( !otherPolygon )
+    return -1;
+
+  if ( mExteriorRing && !otherPolygon->mExteriorRing )
+    return 1;
+  else if ( !mExteriorRing && otherPolygon->mExteriorRing )
+    return -1;
+  else if ( mExteriorRing && otherPolygon->mExteriorRing )
+  {
+    int shellComp = mExteriorRing->compareTo( otherPolygon->mExteriorRing.get() );
+    if ( shellComp != 0 )
+    {
+      return shellComp;
+    }
+  }
+
+  const int nHole1 = mInteriorRings.size();
+  const int nHole2 = otherPolygon->mInteriorRings.size();
+  if ( nHole1 < nHole2 )
+  {
+    return -1;
+  }
+  if ( nHole1 > nHole2 )
+  {
+    return 1;
+  }
+
+  for ( int i = 0; i < nHole1; i++ )
+  {
+    const int holeComp = mInteriorRings.at( i )->compareTo( otherPolygon->mInteriorRings.at( i ) );
+    if ( holeComp != 0 )
+    {
+      return holeComp;
+    }
+  }
+
+  return 0;
+}
